@@ -18,9 +18,9 @@ from __future__ import annotations
 
 import asyncio
 import inspect
-import ipaddress
 import logging
 import random
+import re
 import traceback
 from typing import Any, Callable, Coroutine
 
@@ -84,6 +84,8 @@ IGNORE_MODELS = [
 
 SUPPRESS_CALLBACK_PARAMS = {"SLEEP;EVENT"}
 
+COMMAND_RE = re.compile(r"\((?:\\.|[^(\)\\])*\)")
+
 
 def forceToRange(minv, maxv, val):
     return min(maxv, max(minv, val))
@@ -233,27 +235,22 @@ class SensemeDevice:
 
     def __eq__(self, other: Any) -> bool:
         """Equals magic method."""
-        try:
-            ip_addr = ipaddress.ip_address(other)
-            return ip_addr == ipaddress.ip_address(self._address)
-        except ValueError:
-            pass
         if isinstance(other, SensemeDevice):
-            if self._address and other.address:
-                return self._address == other.address
+            if self._mac and other.mac:
+                return self._mac == other.mac
         if isinstance(other, str):
-            return other in [self._name]
+            return other == self._name
 
         return NotImplemented
 
     def __hash__(self) -> int:
         """Hash magic method."""
-        return hash(self._uuid)
+        return hash(self._mac)  # uuid may not be present via discovery, but mac will
 
     @property
     def is_sec_info_complete(self) -> bool:
         """Return if all secondary information is complete."""
-        return any(val is None for val in self.get_device_info.values())
+        return not any(val is None for val in self.get_device_info.values())
 
     @property
     def get_device_info(self) -> dict[str, Any]:
@@ -712,21 +709,24 @@ class SensemeDevice:
         msg = f"<{self.mac};{cmd}>"
         self._endpoint.send(msg)
 
-    def _process_message(self, line) -> tuple[str, bool]:  # noqa: C901
+    def _process_message(self, line: str) -> tuple[str, bool]:
         """Process messages from device.
 
         May contain multiple responses in one line string.
         Last response may be a partial so it is returned.
         This partial should be added to next line.
         """
+        try:
+            return self._process_message_inner(line)
+        except Exception:  # pylint: disable=broad-except:
+            _LOGGER.warning("Error processing message", exc_info=True)
+            return "", False
+
+    def _process_message_inner(self, line: str) -> tuple[str, bool]:  # noqa: C901
         should_callback = False
         # The line received may have multiple parenthesized responses
         # Split them and process each individual message.
-        # Convert "(msg1)(msg2)(msg3)" to "(msg1)|(msg2)|(msg3)"
-        # then split on the '|'
-        for msg in line.replace(")(", ")|(").split("|"):
-            if msg[-1] != ")":
-                return msg, should_callback
+        for msg in COMMAND_RE.findall(line):
             # remove begining '(' and ending ')' from string
             # also extract name if undefined
             name, result = msg[1:-1].split(";", 1)
